==> jmb.fax-0.3.1.tar.gz: jmb.fax-0.3.1/jmb/fax/kpn/kpn_send.py <==
# coding: utf-8
"""
============
KPN service
============

.. autoclass: KpnManager
"""
import os
import re
import datetime
import logging
import uuid

import pytz
from suds.client  import Client

from django.conf import settings
from jmb.fax import models 
import utils


KPN_URL = 'http://soap.kqumg.it/chaka/current/0.9.5/Chaka.wsdl'
KPN_CB_URL = 'http://soap.kqumg.it/chaka/current/0.9.5/Chaka_CB.wsdl'
# STATUS_URL = 'http://status.kpnqwest.it/'

logger = logging.getLogger('jmb.fax')

class KpnManager(object):
    """
    """
    def __init__(self, username=None, password=None, url=None, device=None, ):
        """Creates a manager to interact with KPN fax service
        
        :enum username: username to provide to the service
        :enum password: password to provide to the service
        :enum url: callbak url. If False non viene impostato (comodo per debug)
        :enum device: a Device instance to be used to find all the previous data. It can 
                      also be a string name that will be used to retrieve the device
        """
        if device:
            if isinstance(device, basestring):
                device = self._get_device(device)
                assert device, "Device %s does not exists" % device
            self.username = device.from_user
            self.password = device.kpn_password
            self.callback_url = None if url == False else device.kpn_url
        else:
            self.username = username
            self.password = password
            self.callback_url = url

        self.callback_url = self.callback_url.rstrip('/') + "/"
        self.suds_client = Client(KPN_URL)
        if self.password:
            self.auth = self.setup()

    def _get_device(self, device_name):
        """Return the device from a name, utility nice from command line
        """

        return models.Device.objects.get(type='kpn', name=device_name)

    def setup(self):

        user_options = self.suds_client.factory.create('UserOptions')
        user_options['contact-person'] = settings.FAX_KPN_CONTACT_PERSON
        if self.callback_url:
            user_options['soap-callback'] = self.callback_url
        auth = self.suds_client.service.start(self.username, self.password, user_options)

        return auth

    def check(self):
        
        try:
            delta = self.suds_client.service.check(self.auth) - datetime.datetime.now(pytz.utc)
            if delta <= datetime.timedelta():
                return False
        except:
            return False
        return True

    def send_message(self, phone_number, filename_list, subject=None, test=False):
        """Send the fax message

        :param phone_number: the recipient number
        :param filename_list: a list of filename paths (or a path) that will be 
                          attached to the fax
        :param subject: a possible subject for the fax
        :param test: boolean. If False just return the message (debug purpose)
        """
        logger.debug("kpn send_message to " + phone_number)
        req_uid = uuid.uuid1().hex
        stored_docs = []
        if isinstance(filename_list, basestring):
            filename_list = [filename_list]
        for filename in filename_list:
            ret, stored_doc = self.store_document(filename)
            stored_docs += [stored_doc]
        docs = self.get_document_list(stored_docs, stored=True)
        msg = self.get_message(phone_number, docs, subject)
        if test:
            return msg
        ret = self.suds_client.service.sendMessage(self.auth, req_uid, msg)
        return ret.info['msg-id']

    def store_document(self, filename):
        source = self.suds_client.factory.create('DocumentSource')
        source['document'], unique_name = self.get_document(filename)
        ret = self.suds_client.service.storeDocument(self.auth, source)
        return ret, unique_name

    def get_document(self, filename):
        doc = self.suds_client.factory.create('Document')
        doc['data'], md5_sign = utils.encode_file(filename)
        doc['unique-name'] = os.path.basename(filename)
        doc['content-type'] = os.path.splitext(os.path.basename(filename))[1].lstrip('.')
        return doc, os.path.basename(filename)

    def get_message(self, destination, document_list, subject):

        fax = self.suds_client.factory.create('FAXMessage')
        fax['documents'] = document_list
        fax['destinations'] = self.get_destination_list(destination)

        msg = self.suds_client.factory.create('Message')
        msg['subject'] = subject
        msg['fax'] = fax

        return msg

    def get_destination_list(self, phone_number):
        destination_list = self.suds_client.factory.create('DestinationList')
        destination_list['item'] = self.get_destination(phone_number)
        return destination_list

    def get_destination(self, phone_number):
        destination = self.suds_client.factory.create('Destination')
        destination['address'] = phone_number
        destination['cc1'] = 'xx'
        destination['cc2'] = 'yy'
        return destination

    def get_document_list(self, filename_list, stored=True):
        """Return a DocumentList of Document or DocumentRef

        :param filename: the filename (or unique_name is stored=True)
        :param stored: boolean. If stored a DocumentRef is used
        """

        docs = self.suds_client.factory.create('DocumentList')
        doc_choice_list = []
        for i, filename in enumerate(filename_list):
            doc_choice = self.suds_client.factory.create('DocumentChoiceElement')
            if stored:
                inner = self.get_document_ref(filename)
                doc_choice['document-ref'] = inner
            else:
                inner = self.get_document(filename)
                doc_choice['document'] = inner
            doc_choice_list += [doc_choice]
        docs['item'] = doc_choice_list
        return docs

    def get_document_ref(self, unique_name):
        doc_ref = self.suds_client.factory.create('DocumentRef')
        doc_ref['unique-name'] = unique_name
        return doc_ref

    def get_status_message_by_date(self, start=None, stop=None):
        start = self.suds_client.factory.create('ns0:dateTime')
        stop = self.suds_client.factory.create('ns0:dateTime')
        raise NotImplementedError

    send_fax = send_message

def send_via_kpn(fax, test=False):
    """
    Send a fax using Kpn service. Cover and documents are read 
    from fax.cover and fax.documents

    :param fax: a models.Fax instance
    :param test: boolean. Just pass test to send_message

    """
    logger.debug("send_via_kpn: id: %s, recipient: %s" % (fax.id, 
                                                          fax.recipient_number))
    docs = [] if not fax.has_cover() else [fax.cover.save_as_pdf(ps=False)]
    docs += [f.file.path for f in fax.documents.all()]
    logger.debug("send_via_kpn: docs: %s" % docs)
    
    k = KpnManager(device=fax.device)

    # Invia
    recipient_number = re.sub('[-()\. /]', '', fax.recipient_number)

    id_msg = k.send_fax(recipient_number, docs, subject=fax.subject, test=test)
    if test:
        return id_msg
    fax.job = id_msg
    fax.status = 9  # Accepted by peer
    logger.debug('send_via_kpn: set status = 9')
    fax.save()

==> jmb.fax-0.6.tar.gz: jmb.fax-0.6/jmb/fax/kpn/kpn_send.py <==
# coding: utf-8
"""
============
KPN service
============

.. autoclass: KpnManager
"""
from __future__ import unicode_literals

import os
import re
import datetime
import logging
import uuid

import pytz
from suds.client import Client

from django.utils import six
from django.conf import settings
from jmb.fax import models
from . import utils


KPN_URL = 'http://soap.kqumg.it/chaka/current/0.9.5/Chaka.wsdl'
KPN_CB_URL = 'http://soap.kqumg.it/chaka/current/0.9.5/Chaka_CB.wsdl'
# STATUS_URL = 'http://status.kpnqwest.it/'

logger = logging.getLogger('jmb.fax')

class KpnManager(object):

    """
    """

    def __init__(self, username=None, password=None, url=None, device=None, ):
        """Creates a manager to interact with KPN fax service

        :enum username: username to provide to the service
        :enum password: password to provide to the service
        :enum url: callbak url. If False non viene impostato (comodo per debug)
        :enum device: a Device instance to be used to find all the previous data. It can 
                      also be a string name that will be used to retrieve the device
        """
        if device:
            if isinstance(device, six.string_types):
                device = self._get_device(device)
                assert device, "Device %s does not exists" % device
            self.username = device.from_user
            self.password = device.kpn_password
            self.callback_url = None if url == False else device.kpn_url
        else:
            self.username = username
            self.password = password
            self.callback_url = url

        self.callback_url = self.callback_url.rstrip('/') + "/"
        self.suds_client = Client(KPN_URL)
        if self.password:
            self.auth = self.setup()

    def _get_device(self, device_name):
        """Return the device from a name, utility nice from command line
        """

        return models.Device.objects.get(type='kpn', name=device_name)

    def setup(self):

        user_options = self.suds_client.factory.create('UserOptions')
        user_options['contact-person'] = settings.FAX_KPN_CONTACT_PERSON
        if self.callback_url:
            user_options['soap-callback'] = self.callback_url
        auth = self.suds_client.service.start(self.username, self.password, user_options)

        return auth

    def check(self):

        try:
            delta = self.suds_client.service.check(self.auth) - datetime.datetime.now(pytz.utc)
            if delta <= datetime.timedelta():
                return False
        except:
            return False
        return True

    def send_message(self, phone_number, filename_list, subject=None, test=False):
        """Send the fax message

        :param phone_number: the recipient number
        :param filename_list: a list of filename paths (or a path) that will be 
                          attached to the fax
        :param subject: a possible subject for the fax
        :param test: boolean. If False just return the message (debug purpose)
        """
        logger.debug("kpn send_message to " + phone_number)
        req_uid = uuid.uuid1().hex
        stored_docs = []
        if isinstance(filename_list, six.string_types):
            filename_list = [filename_list]
        for filename in filename_list:
            ret, stored_doc = self.store_document(filename)
            stored_docs += [stored_doc]
        docs = self.get_document_list(stored_docs, stored=True)
        msg = self.get_message(phone_number, docs, subject)
        if test:
            return msg
        ret = self.suds_client.service.sendMessage(self.auth, req_uid, msg)
        return ret.info['msg-id']

    def store_document(self, filename):
        source = self.suds_client.factory.create('DocumentSource')
        source['document'], unique_name = self.get_document(filename)
        ret = self.suds_client.service.storeDocument(self.auth, source)
        return ret, unique_name

    def get_document(self, filename):
        doc = self.suds_client.factory.create('Document')
        doc['data'], md5_sign = utils.encode_file(filename)
        doc['unique-name'] = os.path.basename(filename)
        doc['content-type'] = os.path.splitext(os.path.basename(filename))[1].lstrip('.')
        return doc, os.path.basename(filename)

    def get_message(self, destination, document_list, subject):

        fax = self.suds_client.factory.create('FAXMessage')
        fax['documents'] = document_list
        fax['destinations'] = self.get_destination_list(destination)

        msg = self.suds_client.factory.create('Message')
        msg['subject'] = subject
        msg['fax'] = fax

        return msg

    def get_destination_list(self, phone_number):
        destination_list = self.suds_client.factory.create('DestinationList')
        destination_list['item'] = self.get_destination(phone_number)
        return destination_list

    def get_destination(self, phone_number):
        destination = self.suds_client.factory.create('Destination')
        destination['address'] = phone_number
        destination['cc1'] = 'xx'
        destination['cc2'] = 'yy'
        return destination

    def get_document_list(self, filename_list, stored=True):
        """Return a DocumentList of Document or DocumentRef

        :param filename: the filename (or unique_name is stored=True)
        :param stored: boolean. If stored a DocumentRef is used
        """

        docs = self.suds_client.factory.create('DocumentList')
        doc_choice_list = []
        for i, filename in enumerate(filename_list):
            doc_choice = self.suds_client.factory.create('DocumentChoiceElement')
            if stored:
                inner = self.get_document_ref(filename)
                doc_choice['document-ref'] = inner
            else:
                inner = self.get_document(filename)
                doc_choice['document'] = inner
            doc_choice_list += [doc_choice]
        docs['item'] = doc_choice_list
        return docs

    def get_document_ref(self, unique_name):
        doc_ref = self.suds_client.factory.create('DocumentRef')
        doc_ref['unique-name'] = unique_name
        return doc_ref

    def get_status_message_by_date(self, start=None, stop=None):
        start = self.suds_client.factory.create('ns0:dateTime')
        stop = self.suds_client.factory.create('ns0:dateTime')
        raise NotImplementedError

    send_fax = send_message

def send_via_kpn(fax, test=False):
    """
    Send a fax using Kpn service. Cover and documents are read 
    from fax.cover and fax.documents

    :param fax: a models.Fax instance
    :param test: boolean. Just pass test to send_message

    """
    logger.debug("send_via_kpn: id: %s, recipient: %s" % (fax.id,
                                                          fax.recipient_number))
    docs = [] if not fax.has_cover() else [fax.cover.save_as_pdf()]
    docs += [f.file.path for f in fax.documents.all()]
    logger.debug("send_via_kpn: docs: %s" % docs)

    k = KpnManager(device=fax.device)

    # Invia
    recipient_number = re.sub('[-()\. /]', '', fax.recipient_number)

    if settings.FAX_FAKE_SEND:
        logger.debug('fake send_via_kpn: set status = 9')
        fax.job = "Fake: send_via_kpn"
        fax.save()
        return

    id_msg = k.send_fax(recipient_number, docs, subject=fax.subject, test=test)
    if test:
        return id_msg
    fax.job = id_msg
    fax.status = 9  # Accepted by peer
    logger.debug('send_via_kpn: set status = 9')
    fax.save()
